@@ -1,6 +1,11 @@
 import datetime
-from collections import OrderedDict
-from mock import Mock, MagicMock, PropertyMock
+import random
+from collections import OrderedDict, namedtuple
+from six import with_metaclass
+try:
+    from unittest.mock import Mock, MagicMock, PropertyMock
+except ImportError:
+    from mock import Mock, MagicMock, PropertyMock
 
 from .constants import *
 from .exceptions import *
@@ -9,7 +14,14 @@ from .utils import (
 )
 
 
-class MockSet(MagicMock):
+class MockSetMeta(type):
+    def __call__(cls, *initial_items, **kwargs):
+        obj = super(MockSetMeta, cls).__call__(**kwargs)
+        obj.add(*initial_items)
+        return obj
+
+
+class MockSet(with_metaclass(MockSetMeta, MagicMock)):
     EVENT_ADDED = 'added'
     EVENT_UPDATED = 'updated'
     EVENT_SAVED = 'saved'
@@ -22,7 +34,8 @@ class MockSet(MagicMock):
         'using',
         'select_related',
         'prefetch_related',
-        'select_for_update'
+        'select_for_update',
+        'iterator'
     ]
 
     def __init__(self, *initial_items, **kwargs):
@@ -104,14 +117,16 @@ class MockSet(MagicMock):
     def filter(self, *args, **attrs):
         results = list(self.items)
         for x in args:
-            if isinstance(x, DjangoQ):
-                results = self._filter_q(results, x)
-            else:
+            if not isinstance(x, DjangoQ):
                 raise ArgumentNotSupported()
+
+            if len(x) > 0:
+                results = self._filter_q(results, x)
+
         return MockSet(*matches(*results, **attrs), clone=self)
 
     def exclude(self, *args, **attrs):
-        excluded = self.filter(*args, **attrs)
+        excluded = set(self.filter(*args, **attrs))
         results = [item for item in self.items if item not in excluded]
         return MockSet(*results, clone=self)
 
@@ -153,6 +168,9 @@ class MockSet(MagicMock):
     def order_by(self, *fields):
         results = self.items
         for field in reversed(fields):
+            if field == '?':
+                random.shuffle(results)
+                break
             is_reversed = field.startswith('-')
             attr = field[1:] if is_reversed else field
             results = sorted(results,
@@ -356,21 +374,36 @@ class MockSet(MagicMock):
                 data.append(values_dict[key])
             return tuple(data)
 
-    def values_list(self, *fields, **kwargs):
+    def _values_row(self, values_dict, fields, **kwargs):
         flat = kwargs.pop('flat', False)
+        named = kwargs.pop('named', False)
 
         if kwargs:
             raise TypeError('Unexpected keyword arguments to values_list: %s' % (list(kwargs),))
         if flat and len(fields) > 1:
             raise TypeError('`flat` is not valid when values_list is called with more than one field.')
-        if len(fields) == 0:
-            raise NotImplementedError('values_list() with no arguments is not implemented')
+        if flat and named:
+            raise TypeError('`flat` and `named` can\'t be used together.')
+
+        if named:
+            Row = namedtuple('Row', fields)
+            row = Row(**values_dict)
+        else:
+            row = self._item_values_list(values_dict, fields, flat)
+
+        return row
+
+    def values_list(self, *fields, **kwargs):
+        # Django doesn't complain about this:
+        # https://github.com/django/django/blob/a4e6030904df63b3f10aa0729b86dc6942b0458e/django/db/models/query.py#L845
+        # if len(fields) == 0:
+        #     raise NotImplementedError('values_list() with no arguments is not implemented')
 
         result = []
         item_values_dicts = list(self.values(*fields))
 
         for values_dict in item_values_dicts:
-            result.append(self._item_values_list(values_dict, fields, flat))
+            result.append(self._values_row(values_dict, fields, **kwargs))
 
         return MockSet(*result, clone=self)
 
